package org.yaoqiang.bpmn.model.elements.core.service;

import java.util.ArrayList;
import java.util.List;

import org.yaoqiang.bpmn.model.BPMNModelUtils;
import org.yaoqiang.bpmn.model.elements.XMLAttribute;
import org.yaoqiang.bpmn.model.elements.XMLElement;
import org.yaoqiang.bpmn.model.elements.XMLTextElement;
import org.yaoqiang.bpmn.model.elements.XMLTextElements;
import org.yaoqiang.bpmn.model.elements.core.common.BPMNError;
import org.yaoqiang.bpmn.model.elements.core.foundation.BaseElement;

/**
 * Operation
 * 
 * @author Shi Yaoqiang(shi_yaoqiang@yahoo.com)
 */
public class Operation extends BaseElement {

	public Operation(Operations parent) {
		super(parent, "operation");
	}

	protected void fillStructure() {
		XMLAttribute attrName = new XMLAttribute(this, "name");
		XMLAttribute attrImplementationRef = new XMLAttribute(this, "implementationRef");
		XMLTextElement refInMessageRef = new XMLTextElement(this, "inMessageRef");
		XMLTextElement refOutMessageRef = new XMLTextElement(this, "outMessageRef");
		XMLTextElements refErrorRef = new XMLTextElements(this, "errorRef");

		super.fillStructure();
		add(attrName);
		add(attrImplementationRef);
		add(refInMessageRef);
		add(refOutMessageRef);
		add(refErrorRef);
	}

	public Operations getParent() {
		return (Operations) parent;
	}

	public final String getName() {
		return get("name").toValue();
	}

	public final String getImplementationRef() {
		return get("implementationRef").toValue();
	}

	public final XMLTextElements getErrorRefs() {
		return (XMLTextElements) get("errorRef");
	}

	public final List<XMLElement> getErrorRefList() {
		return getErrorRefs().getXMLElements();
	}

	public final List<XMLElement> getRefErrorList() {
		List<XMLElement> els = new ArrayList<XMLElement>();
		for (XMLElement erRef : getErrorRefList()) {
			BPMNError er = BPMNModelUtils.getDefinitions(parent).getError(erRef.toValue());
			if (er != null) {
				els.add(er);
			}
		}
		return els;
	}

	public final void setName(String name) {
		set("name", name);
	}

	public final void setImplementationRef(String implementationRef) {
		set("implementationRef", implementationRef);
	}

	public final void setInMessageRef(String inMessageRef) {
		set("inMessageRef", inMessageRef);
	}

	public final void setOutMessageRef(String outMessageRef) {
		set("outMessageRef", outMessageRef);
	}

	public final void addErrorRef(String errorRef) {
		XMLElement error = getErrorRefs().generateNewElement();
		error.setValue(errorRef);
		getErrorRefs().add(error);
	}

}
