package org.yaoqiang.bpmn.model.elements.gateways;

import java.util.ArrayList;
import java.util.List;

import org.yaoqiang.bpmn.model.elements.XMLAttribute;
import org.yaoqiang.bpmn.model.elements.XMLElement;
import org.yaoqiang.bpmn.model.elements.activities.SubProcess;
import org.yaoqiang.bpmn.model.elements.core.common.FlowElements;
import org.yaoqiang.bpmn.model.elements.core.common.FlowNode;
import org.yaoqiang.bpmn.model.elements.process.BPMNProcess;

/**
 * Gateway
 * 
 * @author Shi Yaoqiang(shi_yaoqiang@yahoo.com)
 */
public abstract class Gateway extends FlowNode {

	public Gateway(FlowElements parent, String name) {
		super(parent, name);
	}

	protected void fillStructure() {
		XMLAttribute attrGatewayDirection = new XMLAttribute(this, "gatewayDirection", GatewayDirection.Unspecified.toString());

		super.fillStructure();
		add(attrGatewayDirection);
	}

	public List<XMLElement> getAccessibleProperties() {
		List<XMLElement> properties = new ArrayList<XMLElement>();
		if (getParent().getParent() instanceof SubProcess) {
			properties.addAll(((SubProcess) getParent().getParent()).getAccessibleProperties());
		} else if (getParent().getParent() instanceof BPMNProcess) {
			properties.addAll(((BPMNProcess) getParent().getParent()).getAccessibleProperties());
		}
		return properties;
	}

	public final String getGatewayDirection() {
		return get("gatewayDirection").toValue();
	}
	
	public final void setGatewayDirection(String gatewayDirection) {
		set("gatewayDirection", gatewayDirection);
	}
	
	enum GatewayDirection {

		Unspecified("Unspecified"), Converging("Converging"), Diverging("Diverging"), Mixed("Mixed");

		private String gatewayDirection;

		GatewayDirection(String gatewayDirection) {
			this.setGatewayDirection(gatewayDirection);
		}

		public void setGatewayDirection(String gatewayDirection) {
			this.gatewayDirection = gatewayDirection;
		}

		public String getGatewayDirection() {
			return gatewayDirection;
		}

	}
}
