package org.yaoqiang.graph.editor.swing;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.JWindow;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import org.yaoqiang.graph.editor.GraphEditor;
import org.yaoqiang.graph.editor.action.EditorActions;
import org.yaoqiang.graph.editor.dialog.BaseDialog;
import org.yaoqiang.graph.editor.util.EditorConstants;
import org.yaoqiang.graph.editor.util.EditorUtils;
import org.yaoqiang.graph.editor.view.Graph;
import org.yaoqiang.graph.model.YGraphModel;
import org.yaoqiang.graph.util.Constants;
import org.yaoqiang.graph.view.SwimlaneManager;
import org.yaoqiang.graph.view.YGraphManager;

import com.mxgraph.model.mxCell;
import com.mxgraph.model.mxGraphModel;
import com.mxgraph.swing.mxGraphOutline;
import com.mxgraph.swing.handler.mxRubberband;
import com.mxgraph.util.mxEvent;
import com.mxgraph.util.mxEventObject;
import com.mxgraph.util.mxEventSource.mxIEventListener;
import com.mxgraph.util.mxResources;
import com.mxgraph.util.mxUndoManager;

/**
 * BaseEditor
 * 
 * @author Shi Yaoqiang(shi_yaoqiang@yahoo.com)
 */
public class BaseEditor extends JPanel {

	private static final long serialVersionUID = 1L;

	public static Locale locale = Locale.getDefault();

	protected static File currentFile;

	protected boolean modified = false;

	protected static boolean nativeFormat = false;

	protected String appTitle;

	protected JFrame frame;

	protected BaseDialog dialog;

	protected JSplitPane mainPane;

	protected TabbedPane centerPane;

	protected Map<String, GraphComponent> graphComponents = new HashMap<String, GraphComponent>();

	protected GraphComponent graphComponent;

	protected GraphComponent currentGraphComponent;

	protected Map<String, mxGraphOutline> graphOverviews = new HashMap<String, mxGraphOutline>();

	protected mxGraphOutline graphOverview;

	protected JDialog graphOverviewWindow;

	protected Palettes elementsPalette;

	protected JTabbedPane palettesPane;

	protected JWindow openSplash;

	protected JLabel statusBar;

	protected JMenu recentFilesmenu;

	protected static Graph graph;

	protected mxUndoManager undoManager;

	protected mxIEventListener eventListener = new mxIEventListener() {
		public void invoke(Object sender, mxEventObject evt) {
			graph.clearSelection();
		}
	};

	protected MouseWheelListener wheelTracker = new MouseWheelListener() {
		public void mouseWheelMoved(MouseWheelEvent e) {
			mxGraphModel model = currentGraphComponent.getGraph().getModel();
			if (model.getChildCount(model.getRoot()) > 0 && (e.getSource() instanceof mxGraphOutline || e.isControlDown())) {
				BaseEditor.this.mouseWheelMoved(e);
			}
		}

	};

	protected mxIEventListener groupHandler = new mxIEventListener() {
		public void invoke(Object source, mxEventObject evt) {
			String eventName = evt.getName();
			mxCell cell = (mxCell) graph.getSelectionCell();
			if (cell == null) {
				cell = (mxCell) evt.getProperty("cell");
			}
			if (eventName.equals(Constants.ENTER_GROUP)) {
				String name = BaseEditor.this.getDiagramName();
				if (graph.isSubProcess(cell)) {
					name += " -> SubProcess: " + cell.getValue();
					BaseEditor.this.setDiagramName(name);
					currentGraphComponent.setLastViewRoot(cell);
				}
			} else if (eventName.equals(Constants.EXIT_GROUP)) {
				String name = BaseEditor.this.getDiagramName();
				if (graph.isSubProcess(cell)) {
					int index = name.lastIndexOf(" -> SubProcess: ");
					if (index > 0) {
						name = name.substring(0, index);
					}
				}

				mxCell root = (mxCell) graph.getCurrentRoot();
				Object rootCell = graph.getModel().getCell(currentGraphComponent.getName());
				if (root != rootCell && (root == null || graph.getModel().isAncestor(root, rootCell))) {
					graph.getView().setCurrentRoot(rootCell);
					currentGraphComponent.setLastViewRoot(rootCell);
				} else {
					currentGraphComponent.setLastViewRoot(root);
				}

				if (root == null || graph.getModel().isAncestor(root, rootCell)) {
					BaseEditor.this.resetDiagramName();
					return;
				} else if (graph.isSubProcess(root)) {
					int index = name.lastIndexOf(" -> SubProcess: ");
					if (index == -1 || !name.substring(index + 16).equals(root.getValue())) {
						name += " -> SubProcess: " + root.getValue();
					}
				}
				BaseEditor.this.setDiagramName(name);
			}
		}
	};

	protected void initEditor() {
		setName(getClass().getSimpleName());
		configure();
		initGraphComponent();
		initGraph();
		initGraphOverview();

		dialog = new BaseDialog(this);

		undoManager = new mxUndoManager(graph);

		YGraphManager graphManager = new YGraphManager(currentGraphComponent);
		undoManager.addListener(mxEvent.UNDO, graphManager.getHandler());
		undoManager.addListener(mxEvent.REDO, graphManager.getHandler());

		SwimlaneManager swimlaneManager = new SwimlaneManager(currentGraphComponent);
		undoManager.addListener(mxEvent.UNDO, swimlaneManager.getHandler());
		undoManager.addListener(mxEvent.REDO, swimlaneManager.getHandler());

		palettesPane = new JTabbedPane();
		initMainPalette();
		initArtifactPalette();
	}

	protected void createMainPane(JToolBar toolBar, JComponent leftPane, JComponent rightPane) {
		mainPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, leftPane, rightPane);
		int cols = Integer.parseInt(Constants.SETTINGS.getProperty("paletteCols", "4"));
		if (cols == 1) {
			mainPane.setDividerLocation(80);
		} else if (cols == 2) {
			mainPane.setDividerLocation(140);
		} else if (cols == 3) {
			mainPane.setDividerLocation(cols * 65);
		} else if (cols == 4) {
			mainPane.setDividerLocation(cols * 62);
		} else {
			mainPane.setDividerLocation(cols * 60);
		}
		mainPane.setDividerSize(6);
		mainPane.setBorder(null);

		setLayout(new BorderLayout());
		add(mainPane, BorderLayout.CENTER);
		add(toolBar, BorderLayout.NORTH);

		statusBar = new JLabel(mxResources.get("ready"));
		statusBar.setBorder(BorderFactory.createEmptyBorder(2, 4, 2, 4));
		if (Constants.SETTINGS.getProperty("showStatusBar", "1").equals("1")) {
			add(statusBar, BorderLayout.SOUTH);
		}
	}

	protected void initGraphComponent() {
		graphComponent = createGraphComponent(getClass().getSimpleName());
		currentGraphComponent = graphComponent;
		installGraphComponentHandlers(currentGraphComponent);
		installGraphComponentListeners(currentGraphComponent);
	}

	protected GraphComponent createGraphComponent(String name) {
		GraphComponent graphComponent = null;
		if (graph != null) {
			graphComponent = new GraphComponent(graph);
		} else {
			graphComponent = new GraphComponent(new Graph(new YGraphModel()));
		}
		if (name != null) {
			graphComponent.setName(name);
		}
		return graphComponent;
	}

	protected void initGraph() {
		graph = graphComponent.getGraph();
		graph.addListener(Constants.ENTER_GROUP, groupHandler);
		graph.addListener(Constants.EXIT_GROUP, groupHandler);
		graph.getModel().addListener(mxEvent.CHANGE, new mxIEventListener() {
			public void invoke(Object source, mxEventObject evt) {
				setModified(true);
			}
		});
	}

	protected void initGraphOverview() {
		graphOverview = new mxGraphOutline(currentGraphComponent);
		graphOverview.setTripleBuffered(true);
		installGraphOverviewListeners(graphOverview);
	}

	public void insertGraphComponent(mxCell cell) {
		GraphComponent com = graphComponents.get(cell.getId());
		if (com == null) {
			com = createGraphComponent(cell.getId());
			graphComponents.put(cell.getId(), com);
			com.setLastViewRoot(cell);
		}
		currentGraphComponent = com;
		graph.getView().setCurrentRoot(com.getLastViewRoot());
		graph.clearSelection();

		centerPane.add(centerPane.getTabTitle(cell.getId(), (String) cell.getValue()), com);
		centerPane.setSelectedComponent(com);

		installGraphComponentHandlers(currentGraphComponent);
		installGraphComponentListeners(currentGraphComponent);

		refreshGraphOverview();
	}

	public void removeGraphComponent(String name) {
		centerPane.remove(graphComponents.get(name));
		centerPane.setSelectedComponent(graphComponent);
		graph.clearSelection();
		graphOverviews.remove(name);
		refreshGraphOverview();
	}

	public void closeGraphComponents() {
		for (Map.Entry<String, GraphComponent> entry : graphComponents.entrySet()) {
			centerPane.remove(entry.getValue());
		}
		graphComponents.clear();
		graphOverviews.clear();
		graphComponent.setLastViewRoot(null);
		centerPane.setSelectedComponent(graphComponent);
		graph.getView().setCurrentRoot(null);
		graph.clearSelection();
	}

	public void refreshGraphOverview() {
		mxGraphOutline oldGraphOverview = graphOverview;
		graphOverview = graphOverviews.get(currentGraphComponent.getName());
		if (graphOverview == null) {
			initGraphOverview();
			graphOverviews.put(graphOverview.getName(), graphOverview);
		}

		Component left = mainPane.getLeftComponent();
		if (left instanceof JSplitPane) {
			int loc = ((JSplitPane) left).getDividerLocation();
			((JSplitPane) left).setLeftComponent(graphOverview);
			((JSplitPane) left).setDividerLocation(loc);
		} else if (getGraphOverviewWindow() != null) {
			getGraphOverviewWindow().remove(oldGraphOverview);
			getGraphOverviewWindow().add(graphOverview);
			getGraphOverviewWindow().validate();
			currentGraphComponent.zoomAndCenter();
		}
	}

	protected void installGraphComponentHandlers(GraphComponent graphComponent) {
		new mxRubberband(graphComponent);
	}

	protected void installGraphComponentListeners(final GraphComponent graphComponent) {
		graphComponent.addMouseWheelListener(wheelTracker);

		graphComponent.getGraphControl().addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				mouseReleased(e);
			}

			public void mouseReleased(MouseEvent e) {
				mxGraphModel model = graphComponent.getGraph().getModel();
				if (e.isPopupTrigger() && model.getChildCount(model.getRoot()) > 0) {
					showGraphPopupMenu(e, graphComponent);
				}
			}
		});

		graphComponent.getGraphControl().addMouseMotionListener(new MouseMotionListener() {
			public void mouseDragged(MouseEvent e) {
				mouseLocationChanged(e, graphComponent);
			}

			public void mouseMoved(MouseEvent e) {
				mouseDragged(e);
			}
		});
	}

	protected void installGraphOverviewListeners(mxGraphOutline graphOverview) {
		graphOverview.addMouseWheelListener(wheelTracker);

		graphOverview.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				mouseReleased(e);
			}

			public void mouseReleased(MouseEvent e) {
				if (e.isPopupTrigger()) {
					showOutlinePopupMenu(e);
				}
			}
		});
	}

	protected void initMainPalette() {
		elementsPalette = insertPalette(mxResources.get("elements"));
		elementsPalette.addListener(mxEvent.SELECT, eventListener);

		elementsPalette.addEventTemplate(mxResources.get("startEvent"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/start_event.png")), "startEvent",
				mxResources.get("startEvent"));
		elementsPalette.addEventTemplate(mxResources.get("intermediateEvent"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/intermediate_event.png")), "intermediateEvent",
				mxResources.get("intermediateEvent"));
		elementsPalette.addEventTemplate(mxResources.get("boundaryEvent"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/intermediate_event.png")), "boundaryEvent",
				mxResources.get("boundaryEvent"));
		elementsPalette.addEventTemplate(mxResources.get("endEvent"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/end_event.png")), "endEvent",
				mxResources.get("endEvent"));
		elementsPalette.addGatewayTemplate(mxResources.get("exclusiveGateway"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/gateway.png")), "exclusiveGateway",
				mxResources.get("exclusiveGateway"));
		elementsPalette.addGatewayTemplate(mxResources.get("exclusiveGatewayWithIndicator"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/gateway_exclusive.png")),
				"exclusiveGatewayWithIndicator", mxResources.get("exclusiveGateway"));
		elementsPalette.addGatewayTemplate(mxResources.get("parallelGateway"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/gateway_parallel.png")), "parallelGateway",
				mxResources.get("parallelGateway"));
		elementsPalette.addGatewayTemplate(mxResources.get("inclusiveGateway"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/gateway_inclusive.png")), "inclusiveGateway",
				mxResources.get("inclusiveGateway"));
		elementsPalette.addGatewayTemplate(mxResources.get("complexGateway"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/gateway_complex.png")), "complexGateway",
				mxResources.get("complexGateway"));
		elementsPalette.addGatewayTemplate(mxResources.get("eventGateway"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/gateway_event.png")), "eventGateway",
				mxResources.get("eventGateway"));
		elementsPalette.addGatewayTemplate(mxResources.get("eventGatewayInstantiate"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/gateway_event_instantiate.png")),
				"eventGatewayInstantiate", mxResources.get("eventGatewayInstantiate"));
		elementsPalette.addGatewayTemplate(mxResources.get("parallelEventGateway"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/gateway_event_parallel_instantiate.png")),
				"parallelEventGateway", mxResources.get("parallelEventGateway"));
		elementsPalette.addTaskTemplate(mxResources.get("task"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/task_no.png")), "task", mxResources.get("task"));
		elementsPalette.addTaskTemplate(mxResources.get("taskSend"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/task_send.png")), "taskSend",
				mxResources.get("taskSend"));
		elementsPalette.addTaskTemplate(mxResources.get("taskReceive"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/task_receive.png")), "taskReceive",
				mxResources.get("taskReceive"));
		elementsPalette.addTaskTemplate(mxResources.get("taskService"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/task_service.png")), "taskService",
				mxResources.get("taskService"));
		elementsPalette.addTaskTemplate(mxResources.get("taskUser"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/task_user.png")), "taskUser",
				mxResources.get("taskUser"));
		elementsPalette.addTaskTemplate(mxResources.get("taskManual"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/task_manual.png")), "taskManual",
				mxResources.get("taskManual"));
		elementsPalette.addTaskTemplate(mxResources.get("taskScript"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/task_script.png")), "taskScript",
				mxResources.get("taskScript"));
		elementsPalette.addTaskTemplate(mxResources.get("taskBusinessRule"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/task_business_rule.png")), "taskBusinessRule",
				mxResources.get("taskBusinessRule"));
		elementsPalette.addTaskTemplate(mxResources.get("callActivity"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/call_activity_no.png")), "callActivity",
				mxResources.get("callActivityLabel"));
		elementsPalette.addTaskTemplate(mxResources.get("callUser"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/call_activity_user.png")), "callUser",
				mxResources.get("callActivityLabel"));
		elementsPalette.addTaskTemplate(mxResources.get("callManual"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/call_activity_manual.png")), "callManual",
				mxResources.get("callActivityLabel"));
		elementsPalette.addTaskTemplate(mxResources.get("callScript"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/call_activity_script.png")), "callScript",
				mxResources.get("callActivityLabel"));
		elementsPalette.addTaskTemplate(mxResources.get("callBusinessRule"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/call_activity_business_rule.png")), "callBusinessRule",
				mxResources.get("callActivityLabel"));
		elementsPalette.addTaskTemplate(mxResources.get("callprocess"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/call_activity_subprocess.png")), "callSubprocess",
				mxResources.get("callActivityLabel"));
		elementsPalette.addSubprocessTemplate(mxResources.get("subprocess"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/subprocess.png")), "subprocess",
				mxResources.get("subprocess"));
		elementsPalette.addSubprocessTemplate(mxResources.get("eventSubProcess"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/subprocess_event.png")), "eventSubprocess",
				mxResources.get("eventSubProcess"));
		elementsPalette.addSubprocessTemplate(mxResources.get("tranSubProcess"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/subprocess_transaction.png")), "tranSubprocess",
				mxResources.get("tranSubProcess"));
		elementsPalette.addSubprocessTemplate(mxResources.get("adHocSubProcess"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/subprocess_adhoc.png")), "adHocSubprocess",
				mxResources.get("adHocSubProcess"));
		elementsPalette.addDataobjectTemplate(mxResources.get("dataObject"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/dataobject.png")), "dataobject", "");
		elementsPalette.addDataobjectTemplate(mxResources.get("dataObjects"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/dataobjects.png")), "dataobjects", "");
		elementsPalette.addDataobjectTemplate(mxResources.get("dataInput"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/datainput.png")), "datainput",
				mxResources.get("dataInput"));
		elementsPalette.addDataobjectTemplate(mxResources.get("dataOutput"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/dataoutput.png")), "dataoutput",
				mxResources.get("dataOutput"));
		elementsPalette.addDataobjectTemplate(mxResources.get("dataInputs"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/datainputs.png")), "datainputs",
				mxResources.get("dataInputs"));
		elementsPalette.addDataobjectTemplate(mxResources.get("dataOutputs"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/dataoutputs.png")), "dataoutputs",
				mxResources.get("dataOutputs"));
		elementsPalette.addTemplate(mxResources.get("swimlane"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/swimlane.png")), "swimlane", 990,
				Integer.parseInt(Constants.SETTINGS.getProperty("style_swimlane_height", "200")), mxResources.get("swimlane"));
		elementsPalette.addTemplate(mxResources.get("verticalSwimlane"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/swimlane_vertical.png")), "verticalSwimlane",
				Integer.parseInt(Constants.SETTINGS.getProperty("style_swimlane_height", "200")), 700, mxResources.get("swimlane"));
		elementsPalette.addMessageTemplate(mxResources.get("initiatingMessage"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/initiating_message.png")), "initiatingMessage", "");
		elementsPalette.addMessageTemplate(mxResources.get("nonInitiatingMessage"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/non_initiating_message.png")), "nonInitiatingMessage",
				"");
		elementsPalette.addChoreographyTemplate(mxResources.get("choreographyTask"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/choreography_task.png")), "choreography");
		elementsPalette.addChoreographyTemplate(mxResources.get("subChoreography"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/choreography_subprocess.png")), "subChoreography");
		elementsPalette.addChoreographyTemplate(mxResources.get("callChoreographyTask"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/call_choreography_task.png")), "callChoreographyTask");
		elementsPalette.addChoreographyTemplate(mxResources.get("callChoreography"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/call_choreography.png")), "callChoreography");
		elementsPalette.addConversationTemplate(mxResources.get("conversation"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/conversation.png")), "conversation", "");
		elementsPalette.addConversationTemplate(mxResources.get("subConversation"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/sub_conversation.png")), "subConversation", "");
		elementsPalette.addConversationTemplate(mxResources.get("callConversation"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/call_conversation.png")), "callConversation", "");
		elementsPalette.addConversationTemplate(mxResources.get("callCollaboration"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/call_collaboration.png")), "callCollaboration", "");
		elementsPalette.addDatastoreTemplate(mxResources.get("dataStore"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/datastore.png")), "datastore", "");
		elementsPalette.addTemplate(mxResources.get("annotation"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/annotation.png")), "annotation",
				mxResources.get("annotation"));
		elementsPalette.addTemplate(mxResources.get("group"),
				new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/palettes/group.png")), "group", 400, 250, "");
		elementsPalette.addOrganizationTemplate(mxResources.get("organization"),
				new ImageIcon(Constants.COMPANY_ICON.getImage().getScaledInstance(32, 32, Image.SCALE_SMOOTH)), "organization",
				mxResources.get("organization"));
		elementsPalette.addOrganizationTemplate(mxResources.get("organizationalUnit"),
				new ImageIcon(Constants.OU_ICON.getImage().getScaledInstance(32, 32, Image.SCALE_SMOOTH)), "organizationalUnit",
				mxResources.get("organizationalUnit"));
		elementsPalette.addOrganizationTemplate(mxResources.get("organizationalGroup"),
				new ImageIcon(Constants.GROUP_ICON.getImage().getScaledInstance(32, 32, Image.SCALE_SMOOTH)), "groupOfNames", mxResources.get("organizationalGroup"));
		elementsPalette.addOrganizationTemplate(mxResources.get("organizationalRole"),
				new ImageIcon(Constants.ROLE_ICON.getImage().getScaledInstance(32, 32, Image.SCALE_SMOOTH)), "organizationalRole", mxResources.get("organizationalRole"));
		elementsPalette.addOrganizationTemplate(mxResources.get("organizationalPerson"),
				new ImageIcon(Constants.USER_ICON.getImage().getScaledInstance(32, 32, Image.SCALE_SMOOTH)), "organizationalPerson", mxResources.get("organizationalPerson"));
	}

	protected void initArtifactPalette() {
		File artifactsDir = new File(EditorConstants.YAOQIANG_USER_HOME + File.separator + EditorConstants.YAOQIANG_ARTIFACTS_DIR);

		for (File dir : artifactsDir.listFiles(new FileFilter() {
			public boolean accept(File pathname) {
				return pathname.isDirectory();
			}
		})) {
			Palettes palette = insertPalette(dir.getName());

			for (File f : dir.listFiles(new FilenameFilter() {
				public boolean accept(File dir, String name) {
					return name.toLowerCase().endsWith(".png") || name.toLowerCase().endsWith(".jpeg") || name.toLowerCase().endsWith(".jpg")
							|| name.toLowerCase().endsWith(".gif");
				}
			})) {
				EditorUtils.addArtifact(palette, f.getName(), f.getParent() + File.separator);
			}
		}
	}

	public void setModified(boolean modified) {
		boolean oldValue = this.modified;
		this.modified = modified;

		firePropertyChange("modified", oldValue, modified);

		if (oldValue != modified) {
			updateTitle();
		}
	}

	public boolean isModified() {
		return modified;
	}

	public mxUndoManager getUndoManager() {
		return undoManager;
	}

	public void setCurrentFile(File file) {
		setCurrentFile(file, false);
	}

	public void setCurrentFile(File file, boolean nativeFormat) {
		currentFile = file;
		BaseEditor.nativeFormat = nativeFormat;
		updateTitle();
	}

	public static File getCurrentFile() {
		return currentFile;
	}

	public static boolean isNativeFormat() {
		return nativeFormat;
	}

	public static void setNativeFormat(boolean nativeFormat) {
		BaseEditor.nativeFormat = nativeFormat;
	}

	public JFrame getFrame() {
		return frame;
	}

	public BaseDialog getDialog() {
		return dialog;
	}

	public BaseDialog createDialog() {
		return new BaseDialog(this);
	}

	public JSplitPane getMainPane() {
		return mainPane;
	}

	public TabbedPane getCenterPane() {
		return centerPane;
	}

	public JTabbedPane getPalettesPane() {
		return palettesPane;
	}

	public JLabel getStatusBar() {
		return statusBar;
	}

	public Map<String, GraphComponent> getGraphComponents() {
		return graphComponents;
	}

	public GraphComponent getGraphComponent(String name) {
		return graphComponents.get(name);
	}

	public GraphComponent getGraphComponent() {
		return graphComponent;
	}

	public JDialog getGraphOverviewWindow() {
		return graphOverviewWindow;
	}

	public mxGraphOutline getGraphOverviewComponent() {
		return graphOverview;
	}

	public final JMenu getRecentFilesmenu() {
		return recentFilesmenu;
	}

	public final void setRecentFilesmenu(JMenu recentFilesmenu) {
		this.recentFilesmenu = recentFilesmenu;
	}

	public GraphComponent getCurrentGraphComponent() {
		return currentGraphComponent;
	}

	public void setCurrentGraphComponent(GraphComponent currentGraphComponent) {
		this.currentGraphComponent = currentGraphComponent;
	}

	protected void showOutlinePopupMenu(MouseEvent e) {
		Point pt = SwingUtilities.convertPoint(e.getComponent(), e.getPoint(), currentGraphComponent);

		JCheckBoxMenuItem item = new JCheckBoxMenuItem(mxResources.get("showLabels"));
		item.setSelected(graphOverview.isDrawLabels());

		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphOverview.setDrawLabels(!graphOverview.isDrawLabels());
				graphOverview.repaint();
			}
		});

		JPopupMenu menu = new JPopupMenu();
		menu.add(item);
		menu.show(currentGraphComponent, pt.x, pt.y);

		e.consume();
	}

	protected void mouseWheelMoved(MouseWheelEvent e) {
		if (e.getWheelRotation() < 0) {
			currentGraphComponent.zoomIn();
		} else {
			currentGraphComponent.zoomOut();
		}

		status(mxResources.get("scale") + ": " + (int) (100 * currentGraphComponent.getGraph().getView().getScale()) + "%");
	}

	protected void showGraphPopupMenu(MouseEvent e, GraphComponent graphComponent) {
		Point pt = SwingUtilities.convertPoint(e.getComponent(), e.getPoint(), graphComponent);
		JPopupMenu menu = createGraphPopupMenu(graphComponent);
		graphComponent.setPopupPoint(e.getPoint());
		menu.show(graphComponent, pt.x, pt.y);
		e.consume();
	}

	protected JPopupMenu createGraphPopupMenu(GraphComponent graphComponent) {
		JPopupMenu menu = new BasePopupMenu(this, graphComponent);
		return menu;
	}

	protected void mouseLocationChanged(MouseEvent e, GraphComponent graphComponent) {
		status(mxResources.get("scale") + ": " + (int) (100 * graphComponent.getGraph().getView().getScale()) + "%" + ", " + mxResources.get("location")
				+ ": x=" + e.getX() + ", y=" + e.getY());
	}

	public void status(String msg) {
		statusBar.setText(msg);
	}

	public void updateTitle() {
		Window window = SwingUtilities.windowForComponent(this);
		if (window != null && window instanceof JFrame) {
			String title = (currentFile != null) ? currentFile.getAbsolutePath() : mxResources.get("newDiagram");

			if (nativeFormat) {
				title += " (" + mxResources.get("nativeFormat") + ")";
			}
			if (modified) {
				title += "*";
			}
			((JFrame) window).setTitle(title + " - " + appTitle);
		}
	}

	public void progress(boolean start) {
		Window window = SwingUtilities.windowForComponent(this);
		if (Constants.OS.startsWith("Windows")) {
			if (start) {
				openSplash = new JWindow(window);
				openSplash.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				currentGraphComponent.getGraphControl().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				JPanel panel = new JPanel();
				ImageIcon imageIcon = new ImageIcon(GraphEditor.class.getResource("/org/yaoqiang/graph/editor/images/loading.gif"));
				panel.add(new JLabel(mxResources.get("opening"), imageIcon, JLabel.LEFT));
				panel.setBorder(BorderFactory.createEmptyBorder(10, 20, 10, 20));
				openSplash.add(panel);
				openSplash.pack();
				openSplash.setLocationRelativeTo(window);
				openSplash.setVisible(true);
			} else {
				currentGraphComponent.getGraphControl().setCursor(null);
				if (openSplash != null) {
					openSplash.setCursor(null);
					openSplash.dispose();
					openSplash = null;
				}
			}
		}
	}

	public Palettes insertPalette(String title) {
		final Palettes palette = new Palettes(title);
		final JScrollPane scrollPane = new JScrollPane(palette);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		palettesPane.add(title, scrollPane);

		palettesPane.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				int w = scrollPane.getWidth() - scrollPane.getVerticalScrollBar().getWidth();
				palette.setPreferredWidth(w);
			}

		});

		return palette;
	}

	public void createGraphOverviewWindow(Frame window, mxGraphOutline graphOutline) {
		if (Constants.SETTINGS.getProperty("showFloatingOutline", Constants.OS.startsWith("Windows") ? "1" : "0").equals("1")) {
			graphOverviewWindow = new JDialog(window, mxResources.get("Outline"));
			graphOverviewWindow.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					BaseMenuBar.getOutlineMenuItem().setSelected(false);
				}
			});
			graphOverviewWindow.setModal(false);
			graphOverviewWindow.add(graphOutline);

			int x = window.getX();
			int y = window.getY() + (3 * window.getHeight() / 4);
			graphOverviewWindow.setLocation(x, y);
			graphOverviewWindow.setSize(250, 220);

			graphOverviewWindow.setVisible(Constants.SETTINGS.getProperty("showOutline", "1").equals("1"));
		}
	}

	public void removeGraphOverviewWindow() {
		graphOverviewWindow.dispose();
		graphOverviewWindow = null;
	}

	public final class WindowCloser extends WindowAdapter {
		public void windowClosing(WindowEvent e) {
			int o = JOptionPane.NO_OPTION;
			if (isModified()) {
				o = JOptionPane.showConfirmDialog(null, mxResources.get("saveChanges"), mxResources.get("optionTitle"), JOptionPane.YES_NO_CANCEL_OPTION);
				if (o == JOptionPane.YES_OPTION) {
					EditorActions.getSaveAction().actionPerformed(new ActionEvent(getGraphComponent(), e.getID(), null));
				}
			}
			if (!isModified() || o == JOptionPane.NO_OPTION) {
				System.exit(0);
			}
		}
	}

	public void installWindowListener() {
		Window window = SwingUtilities.windowForComponent(this);
		if (window != null) {
			window.addWindowListener(new GraphEditor.WindowCloser());
		}
	}

	public JFrame createFrame(JMenuBar menuBar) {
		JFrame frame = new JFrame();
		frame.setIconImage(new ImageIcon(GraphEditor.class.getClassLoader().getResource("org/yaoqiang/graph/images/logo.png")).getImage());
		frame.getContentPane().add(this);
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		frame.setJMenuBar(menuBar);

		Dimension screenSize = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().getBounds().getSize();
		int xMinus = 20, yMinus = 60;
		frame.setBounds(xMinus / 2, yMinus / 2, screenSize.width - xMinus, screenSize.height - yMinus);

		installWindowListener();

		createGraphOverviewWindow(frame, graphOverview);

		updateTitle();

		return frame;
	}

	public void showFrame(boolean visible) {
		frame.setVisible(visible);
	}

	public void setDiagramName(String name) {
		centerPane.setTabTitle(currentGraphComponent.getName(), name);
	}

	public String getDiagramName() {
		return centerPane.getTabTitle(currentGraphComponent.getName(), "");
	}

	public void resetDiagramName() {
		String name = null;
		if ("GraphEditor".equals(currentGraphComponent.getName())) {
			name = mxResources.get("newDiagram");
			if (currentFile != null) {
				name = currentFile.getName();
			}
			centerPane.setTitleAt(0, name);
		} else {
			name = graph.getCellValue(currentGraphComponent.getName());
			centerPane.setTabTitle(currentGraphComponent.getName(), name);
		}
	}

	public void setLookAndFeel(String clazz) {
		Window window = SwingUtilities.windowForComponent(this);
		if (window != null) {
			try {
				UIManager.setLookAndFeel(clazz);
				EditorUtils.saveToConfigureFile("Theme", clazz);
				SwingUtilities.updateComponentTreeUI(window);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
		}
	}

	public void about() {
		Window window = SwingUtilities.windowForComponent(this);

		if (window != null && window instanceof Frame) {
			AboutDialog about = new AboutDialog((Frame) window);
			about.setModal(true);

			// Centers inside the application frame
			about.setLocationRelativeTo(window);

			// Shows the modal dialog and waits
			about.setVisible(true);
		}
	}

	public static boolean invalidJRE() {
		boolean invalid = false;
		String jre = System.getProperty("java.runtime.name", "");
		invalid = jre.startsWith("OpenJDK");
		if (invalid) {
			JOptionPane.showMessageDialog(null, "An incompatible Java version has been detected which has been reported to cause strange bugs. Aborting now. \nPlease install the latest release of the Oracle Java SE Runtime Environment (JRE) from: http://java.oracle.com or if it is already installed, make sure it is used.", "Java Version Incompatible", JOptionPane.ERROR_MESSAGE);
		}
		return invalid;
	}
	
	public void restart() {
		Window window = SwingUtilities.windowForComponent(this);
		if (window instanceof JFrame) {
			window.dispose();
			configure();
		}
	}

	public void exit() {
		Window window = SwingUtilities.windowForComponent(this);
		if (window != null) {
			window.dispose();
			System.exit(0);
		}
	}

	public void openFile(String filepath) {
		EditorActions.getOpenAction(filepath).actionPerformed(new ActionEvent(graphComponent, 0, ""));
	}

	public Action bind(String name, final Action action) {
		return bind(name, action, null);
	}

	public Action bind(String name, final Action action, String iconUrl) {
		ImageIcon icon = (iconUrl != null) ? new ImageIcon(BaseEditor.class.getResource(iconUrl)) : null;
		if (icon != null) {
			if (icon.getIconWidth() > 18 || icon.getIconHeight() > 18) {
				icon = new ImageIcon(icon.getImage().getScaledInstance(18, 18, 0));
			}
		}
		Action a = new AbstractAction(name, icon) {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				action.actionPerformed(new ActionEvent(getCurrentGraphComponent(), e.getID(), e.getActionCommand()));
			}
		};
		a.setEnabled(action.isEnabled());
		a.putValue(Action.SHORT_DESCRIPTION, name);
		if (name.equals(mxResources.get("new"))) {
			a.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control N"));
		} else if (name.equals(mxResources.get("openFile"))) {
			a.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control O"));
		} else if (name.equals(mxResources.get("save"))) {
			a.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control S"));
		} else if (name.equals(mxResources.get("saveAs"))) {
			a.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control shift S"));
		} else if (name.equals(mxResources.get("undo"))) {
			a.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control Z"));
		} else if (name.equals(mxResources.get("redo"))) {
			a.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control Y"));
		} else if (name.equals(mxResources.get("selectAll"))) {
			a.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control A"));
		} else if (name.equals(mxResources.get("cut"))) {
			a.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control X"));
		} else if (name.equals(mxResources.get("copy"))) {
			a.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control C"));
		} else if (name.equals(mxResources.get("paste"))) {
			a.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control V"));
		} else if (name.equals(mxResources.get("print"))) {
			a.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control P"));
		} else if (name.equals(mxResources.get("delete"))) {
			a.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("DELETE"));
		}
		return a;
	}

	public void configure() {
		File udir = new File(EditorConstants.YAOQIANG_USER_HOME);
		if (!udir.exists()) {
			try {
				udir.mkdir();
			} catch (Exception exc) {
			}
		}

		File adir = new File(EditorConstants.YAOQIANG_USER_HOME + File.separator + EditorConstants.YAOQIANG_ARTIFACTS_DIR);
		if (!adir.exists()) {
			try {
				adir.mkdir();
			} catch (Exception exc) {
			}
		}

		Constants.SETTINGS = EditorUtils.loadProperties(EditorConstants.YAOQIANG_CONF_FILE);

		String locstring = Constants.SETTINGS.getProperty("Locale", "en");
		if (locstring.equals("zh_CN")) {
			locale = new Locale("zh", "CN");
		} else {
			locale = new Locale(locstring);
		}

		try {
			String def_theme = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";
			if (locstring.equals("zh_CN") && !Constants.OS.startsWith("Windows")) {
				def_theme = "javax.swing.plaf.metal.MetalLookAndFeel";
			}
			String theme = Constants.SETTINGS.getProperty("Theme", def_theme);
			try {
				UIManager.setLookAndFeel(theme);
			} catch (Exception ex) {
				UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
			}
			javax.swing.plaf.FontUIResource f;
			try {
				if (locstring.equals("zh_CN")) {
					f = new javax.swing.plaf.FontUIResource("AR PL UMing CN", Font.TRUETYPE_FONT, 12);
				} else {
					f = new javax.swing.plaf.FontUIResource("Sans Serif", Font.TRUETYPE_FONT, 12);
				}
			} catch (Exception ex) {
				f = new javax.swing.plaf.FontUIResource("Label.font", Font.PLAIN, 12);
			}
			java.util.Enumeration<?> keys = UIManager.getDefaults().keys();
			while (keys.hasMoreElements()) {
				Object key = keys.nextElement();
				Object value = UIManager.get(key);
				if (value instanceof javax.swing.plaf.FontUIResource) {
					UIManager.put(key, f);
				}
			}
		} catch (Exception e1) {
		}

	}

}
